`timescale 1ns/1ps
`define  FILE_NAME    "./header_port1.txt"

module differential_signal_processor_tb();
	reg		clk			;
	reg		rst_n			;
	reg		dp			;
	reg		dn			;
	wire		syncword_vld		;
	wire [7:0]	parallel_data		;
	wire		parallel_data_vld	;

differential_signal_processor differential_signal_processor(
    .clk                 (clk)                  ,      //system clock
    .rst_n               (rst_n)                ,      //async reset
    .dp                  (dp)                   ,      //data positive
    .dn                  (dn)                   ,      //data negative
    .syncword_vld        (syncword_vld)         ,      //syncword valid
    .parallel_data       (parallel_data)        ,      //output data except for syncword
    .parallel_data_vld   (parallel_data_vld)           //output data valid
);

initial begin
	clk = 1;
	forever #5 clk = ~clk;
end

reg	[7:0]	test_data	[0:500] ;	//memory, cache test data
reg	[7:0]	num_test_bytes	        ;	//number of valid data included id
reg	[7:0]	num_zero_bytes		;	//number of zero before the 1d

//read data from txt file to memory
integer i, error_cnt;
initial begin
	error_cnt = 0;
	$readmemh(`FILE_NAME,test_data);
end

//===================task for transmitter=====================//
//send zero before the 1d
task send_zero_data;
	input [7:0] zero_data;
	integer m;
	begin
		for(m=7;m>=0;m=m-1)begin
			dp = zero_data[m];
			dn = ~zero_data[m];
			@(posedge clk);		//each clock send one bit
		end
		$display($time,,,"TX sends data is %0h",zero_data);
	end
endtask

//send multiple zero in a row
task send_multiple_zero;
	input	integer num_bytes;
	integer n;
	begin
		$display($time,,,"Starting to send %0d bytes zero...",num_bytes);
		for(n=0;n<num_bytes;n=n+1)begin
			send_zero_data(8'h0);
		end
		$display($time,,,"Finished sending %0d bytes zero...",num_bytes);
	end
endtask

//parallel to serial
task send_serial_data;
	input [7:0] data;
	integer j;
	begin
		for(j=7;j>=0;j=j-1)begin
			dp = data[j];
			dn = ~data[j];
			@(posedge clk);
		end
		$display($time,,,"TX sends data is %0h",data);
	end
endtask

//send multiple bytes in a row
task send_multiple_bytes;
	input integer num_bytes;
	integer k;
	begin
		$display($time,,,"Starting to send %0d bytes...",num_bytes);
		for(k=0;k<num_bytes;k=k+1)begin
			send_serial_data(test_data[k]);
		end
		$display($time,,,"Finished sending %0d bytes...",num_bytes);
	end
endtask

//===============task   end===================//

reg [7:0]cnt,cnt_r;
always@(posedge clk or negedge rst_n)begin
	if(!rst_n)
		cnt_r <= #1 8'h0;
	else
		cnt_r <= #1 cnt;
end
always@(*)begin
	if(!syncword_vld)
		cnt = 8'h0;
	else if(parallel_data_vld)
		cnt = cnt_r + 1;
	else
		cnt = cnt_r;
end

//check output is pass or fail
wire data_pass = (parallel_data==test_data[cnt]&&parallel_data_vld&&syncword_vld) ? 1'b1:1'b0;
wire data_fail = (parallel_data!=test_data[cnt]&&parallel_data_vld&&syncword_vld) ? 1'b1:1'b0;

always@(posedge clk or negedge rst_n)begin
	if(data_pass) begin
		$display($time,,,"PASS, parallel_data is %0h, expected_data is %0h, output MATCH!!!!",parallel_data,test_data[cnt]);
	end
	else if(data_fail)begin
		$display($time,,,"ERROR, parallel_data is %0h, expected_data is %0h, output mismatch!!!!",parallel_data,test_data[cnt]);
		error_cnt <= error_cnt + 1;
	end
end

initial begin

	rst_n = 1;
	fork
		dp = 1;
		dn = 1;
		num_test_bytes = {$random} % 250;	//the value can be charged
	    num_zero_bytes = {$random} % 20;		//the value can be charged
	join
#20;
	rst_n = 0;
#20;
	rst_n = 1;
#100;
	fork
		dp = 0;
		dn = 0;
	join
#10;

	send_multiple_zero(num_zero_bytes);
	fork
		send_multiple_bytes(num_test_bytes);
		//check_multiple_bytes(num_test_bytes);
	join

		dp = 0;
		dn = 0;
#20;
		dp = 1;
		dn = 1;
#20;
	$display($time,,,"**********************************************************************");
	$display($time,,,"**********************************************************************");
	$display($time,,,"**********************************************************************");
	$display($time,,,"****Finished sending %0d bytes, total errors are %0d****",num_test_bytes,error_cnt);
	$display($time,,,"**********************************************************************");
	$display($time,,,"**********************************************************************");
	$display($time,,,"**********************************************************************");
#50000;
$finish;
end


endmodule
